{
Internal Pascal declarations of the GPC run time system that are not
needed in normal programs. This file is a supplement to gpc.pas. Use
of the routines in this file is at your own risk. They might only
work under certain circumstances, and might change without notice.

Note about the `GPC_' prefix:
This is inserted so that some identifiers don't conflict with the
built-in ones. Sometimes, the built-in ones do exactly the same as
the ones declared here, but often enough, they contain some "magic",
so they should be used instead of the plain declarations here.
In general, routines with a `GPC_' prefix should not be called from
programs. They may change or disappear in future GPC versions.

Copyright (C) 1998-99 Free Software Foundation, Inc.

Author: Frank Heckenbach <frank@pascal.gnu.de>

This file is part of GNU Pascal Library. The GNU Pascal Library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any
later version.

The GNU Pascal Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU Pascal Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.
}

unit Internal;

interface

uses GPC;

{ RTS initialization, called before entering the main Pascal program, from rt0.c }

procedure GPC_Initialize (ArgumentCount : Integer; Arguments, StartEnvironment : PCStrings); asmname '_p_initialize';

{ Subroutines of GPC_Initialize. All GPC_Initialize does is call these routines. }

procedure GPC_Init_Heap;                                                                asmname '_p_init_heap';
procedure GPC_Init_Arguments (ArgumentCount : Integer; Arguments : PCStrings);          asmname '_p_init_arguments';
procedure GPC_Init_Locale;                                                              asmname '_p_init_locale';
procedure GPC_Init_Time;                                                                asmname '_p_init_time';
procedure GPC_Init_Signals;                                                             asmname '_p_init_signals';
procedure GPC_Initialize_Std_Files; { Without I/O checking }                            asmname '_p_initialize_std_files';
procedure GPC_Init_Std_Files;       { With    I/O checking }                            asmname '_p_init_std_files';
procedure GPC_Init_Files_AtExit;                                                        asmname '_p_init_files_atexit';
procedure GPC_Run_Constructors;                                                         asmname '_p_run_constructors';

{ Automatically called at the end of the program }
procedure GPC_Done_Files;                                                               asmname '_p_done_files';

var
  RTSWarnFlag  : asmname '_p_warn'  Integer;
  RTSDebugFlag : asmname '_p_debug' Integer;

procedure FlushAllFiles (OnlyTTYs : Boolean); asmname '_p_fflush';
procedure ExitProgram (Status : Integer); asmname '_p_exit';

{ Heap manager initialization, from heap.pas. Called by GPC_Init_Heap }
procedure HeapInit; asmname '_p_heap_init';

{ String handling routines, from string.c }

function  GPC_String_Operations (Opcode, ArgumentMask : Integer; ...) : Integer;    asmname '_p_string';
procedure GPC_WriteStr          (StringType : Integer; Str1 : PChars; ...);         asmname '_p_writestr';
procedure GPC_ReadStr           (Source : PChars; StrLength, Count : Integer; ...); asmname '_p_readstr';

{ bind a filename to an external file, from files.pas }

procedure GPC_Internal_Assign (var T : AnyFile; Name : CString; NameLength : Integer); asmname '_p_internal_assign';

{ Generic file handling routines and their support, from file.c }

type
  InternalIOSelectTypePtr = ^InternalIOSelectType;
  InternalIOSelectType = record
    fi : PAnyFile;
    WantedReadOrEOF,
    WantedRead,
    WantedEOF,
    WantedWrite,
    WantedException,
    WantedAlways,
    OccurredReadOrEOF,
    OccurredRead,
    OccurredEOF,
    OccurredWrite,
    OccurredException : Boolean
  end;

function  InternalIOSelect (Events : InternalIOSelectTypePtr; EventsLow, Count : Integer; MicroSeconds : MicroSecondTimeType) : Integer; asmname '_p_select';
procedure GPC_InitFDR (var aFile : AnyFile; Name : CString; Size, Flags : Integer); asmname '_p_initfdr';
procedure GPC_DoneFDR (var aFile : AnyFile);                                        asmname '_p_donefdr';

{ Default TFDD routines for files. @@ OpenFunc and CloseProc still missing. }
function  F_Read     (var aFile; var   Buffer; Size : SizeType) : SizeType; asmname '_p_f_read';
function  F_Read_TTY (var aFile; var   Buffer; Size : SizeType) : SizeType; asmname '_p_f_read_tty';
function  F_Write    (var aFile; const Buffer; Size : SizeType) : SizeType; asmname '_p_f_write';
procedure F_Flush    (var aFile);                                           asmname '_p_f_flush';

{ Routines to read various things from files, from read.c }

procedure GPC_TextRead   (var aFile : Text; Count : Integer; ...); asmname '_p_read';
procedure GPC_LazyTryGet (var aFile : AnyFile);                    asmname '_p_lazytryget';
procedure GPC_LazyGet    (var aFile : AnyFile);                    asmname '_p_lazyget';
procedure GPC_LazyUnget  (var aFile : AnyFile);                    asmname '_p_lazyunget';

{ If an error occurs or remaining characters (including whitespace!) are left,
  the position of the error, not the error code, is returned (BP compatibility!) }
function GPC_Val_ByteInt_NoCheck   (Source : PChars; StrLength, Flags : Integer; var Dest : ByteInt  ) : Integer; asmname '_p_val_byteint_nocheck';
function GPC_Val_ShortInt_NoCheck  (Source : PChars; StrLength, Flags : Integer; var Dest : ShortInt ) : Integer; asmname '_p_val_shortint_nocheck';
function GPC_Val_Integer_NoCheck   (Source : PChars; StrLength, Flags : Integer; var Dest : Integer  ) : Integer; asmname '_p_val_integer_nocheck';
function GPC_Val_MedInt_NoCheck    (Source : PChars; StrLength, Flags : Integer; var Dest : MedInt   ) : Integer; asmname '_p_val_medint_nocheck';
function GPC_Val_LongInt_NoCheck   (Source : PChars; StrLength, Flags : Integer; var Dest : LongInt  ) : Integer; asmname '_p_val_longint_nocheck';
function GPC_Val_ByteCard_NoCheck  (Source : PChars; StrLength, Flags : Integer; var Dest : ByteCard ) : Integer; asmname '_p_val_bytecard_nocheck';
function GPC_Val_ShortCard_NoCheck (Source : PChars; StrLength, Flags : Integer; var Dest : ShortCard) : Integer; asmname '_p_val_shortcard_nocheck';
function GPC_Val_Cardinal_NoCheck  (Source : PChars; StrLength, Flags : Integer; var Dest : Cardinal ) : Integer; asmname '_p_val_cardinal_nocheck';
function GPC_Val_MedCard_NoCheck   (Source : PChars; StrLength, Flags : Integer; var Dest : MedCard  ) : Integer; asmname '_p_val_medcard_nocheck';
function GPC_Val_LongCard_NoCheck  (Source : PChars; StrLength, Flags : Integer; var Dest : LongCard ) : Integer; asmname '_p_val_longcard_nocheck';
{$if 0} { Not yet built-in }
function GPC_Val_ByteInt_Check     (Source : PChars; StrLength, Flags : Integer; var Dest : ByteInt;   MinRange, MaxRange : ByteInt  ) : Integer; asmname '_p_val_byteint_check';
function GPC_Val_ShortInt_Check    (Source : PChars; StrLength, Flags : Integer; var Dest : ShortInt;  MinRange, MaxRange : ShortInt ) : Integer; asmname '_p_val_shortint_check';
function GPC_Val_Integer_Check     (Source : PChars; StrLength, Flags : Integer; var Dest : Integer;   MinRange, MaxRange : Integer  ) : Integer; asmname '_p_val_integer_check';
function GPC_Val_MedInt_Check      (Source : PChars; StrLength, Flags : Integer; var Dest : MedInt;    MinRange, MaxRange : MedInt   ) : Integer; asmname '_p_val_medint_check';
function GPC_Val_LongInt_Check     (Source : PChars; StrLength, Flags : Integer; var Dest : LongInt;   MinRange, MaxRange : LongInt  ) : Integer; asmname '_p_val_longint_check';
function GPC_Val_ByteCard_Check    (Source : PChars; StrLength, Flags : Integer; var Dest : ByteCard;  MinRange, MaxRange : ByteCard ) : Integer; asmname '_p_val_bytecard_check';
function GPC_Val_ShortCard_Check   (Source : PChars; StrLength, Flags : Integer; var Dest : ShortCard; MinRange, MaxRange : ShortCard) : Integer; asmname '_p_val_shortcard_check';
function GPC_Val_Cardinal_Check    (Source : PChars; StrLength, Flags : Integer; var Dest : Cardinal;  MinRange, MaxRange : Cardinal ) : Integer; asmname '_p_val_cardinal_check';
function GPC_Val_MedCard_Check     (Source : PChars; StrLength, Flags : Integer; var Dest : MedCard;   MinRange, MaxRange : MedCard  ) : Integer; asmname '_p_val_medcard_check';
function GPC_Val_LongCard_Check    (Source : PChars; StrLength, Flags : Integer; var Dest : LongCard;  MinRange, MaxRange : LongCard ) : Integer; asmname '_p_val_longcard_check';
{$endif}
function GPC_Val_ShortReal         (Source : PChars; StrLength, Flags : Integer; var Dest : ShortReal) : Integer; asmname '_p_val_shortreal';
function GPC_Val_Real              (Source : PChars; StrLength, Flags : Integer; var Dest : Real     ) : Integer; asmname '_p_val_real';
function GPC_Val_LongReal          (Source : PChars; StrLength, Flags : Integer; var Dest : LongReal ) : Integer; asmname '_p_val_longreal';

{ Routines to output various things, from write.c }

procedure GPC_TextWrite (var aFile : Text; Count : Integer; ...); asmname '_p_write';

{ Internal declarations for built-in functionality }

var
  { BP compatible InOutRes variable }
  GPC_InOutRes : asmname '_p_inoutres' Integer;

  { CString parameter to some error messages, NOT the text of the error
    message (this can be obtained with GetErrorMessage) }
  GPC_InOutResStr : asmname '_p_inoutres_str' CString;

procedure GPC_RunError                    (n : Integer);                             asmname '_p_runerror';
procedure StartTempIOError;                                                          asmname '_p_start_temp_io_error';
function  EndTempIOError : Integer;                                                  asmname '_p_end_temp_io_error';
function  GPC_IOResult : Integer;                                                    asmname '_p_ioresult';
procedure GPC_Halt (aExitCode : Integer);                                            asmname '_p_halt';

{ For GNU malloc }
procedure HeapWarning                     (s : CString);                             asmname '_p_heap_warning';

{ For signal handlers }
procedure PrintMessage (Message : CString; n : Integer; Warning : Boolean);          asmname '_p_prmessage';

{ `MoveLeft', `MoveRight' and `Move' transfer procedures, from move.pas }

procedure GPC_MoveLeft  (const Source; var Dest; Count : SizeType); asmname '_p_moveleft';
procedure GPC_MoveRight (const Source; var Dest; Count : SizeType); asmname '_p_moveright';
procedure GPC_Move      (const Source; var Dest; Count : SizeType); asmname '_p_move';

{ GetMemPtr and FreeMemPtr point to these routines by default }
function  Default_GetMem  (Size : SizeType) : Pointer; asmname '_p_default_malloc';
procedure Default_FreeMem (aPointer : Pointer);        asmname 'free';

{ GPC_GetMem and GPC_FreeMem call the actual routines through GetMemPtr and FreeMemPtr }
function  GPC_GetMem   (Size : SizeType) : Pointer;    asmname '_p_malloc';
procedure GPC_FreeMem  (aPointer : Pointer);           asmname '_p_free';

{ GPC_New and GPC_Dispose call GPC_GetMem and GPC_FreeMem, but also do the
  stuff necessary for Mark and Release. Therefore, GPC_GetMem and GPC_FreeMem
  should not be called directly when using Mark and Release. GetMem and
  FreeMem in a Pascal program will call GPC_New and GPC_Dispose internally,
  not GPC_GetMem and GPC_FreeMem. }
function  GPC_New      (Size : SizeType) : Pointer;    asmname '_p_new';
procedure GPC_Dispose  (aPointer : Pointer);           asmname '_p_dispose';

procedure GPC_Mark     (var aPointer : Pointer);       asmname '_p_mark';
procedure GPC_Release  (aPointer : Pointer);           asmname '_p_release';

type
  GPC_PString = ^GPC_String;
  GPC_String (Capacity : Cardinal) = record
    Length : 0 .. Capacity;
    Chars  : packed array [1 .. Capacity + 1] of Char
  end;

  (* Not yet implemented
  StringObject = object { Abstract }
    function  GetCapacity : Integer;           virtual;
    procedure SetLength (NewLength : Integer); virtual;
    function  GetLength : Integer;             virtual;
    function  GetFirstChar : PChars;           virtual;
  end;
  *)

  (*@@ AnyString parameters are not yet implemented, the following is only a draft*)
  AnyStringType = (AnyStringLong,
                   AnyStringUndiscriminated,
                   AnyStringShort,
                   AnyStringFixed,
                   AnyStringCString,
                   AnyStringObject);

  (*@@ only formally for now*)UndiscriminatedString = ^String;

  VarAnyString = record
    Capacity : Integer;
    Chars    : PChars;
    Truncate : Boolean;
  case StringType            : AnyStringType of
    AnyStringLong            : (PLongLength            : ^Integer);
    AnyStringUndiscriminated : (PUndiscriminatedString : ^UndiscriminatedString);
    AnyStringShort           : (PShortLength           : ^Byte);
    AnyStringFixed           : (CurrentLength          : Integer);
    (* AnyStringObject          : (PStringObject          : ^StringObject) *)
  end;

function VarAnyStringLength (var s : VarAnyString) : Integer; asmname '_p_var_anystring_length';
function VarAnyStringSetLength (var s : VarAnyString; NewLength : Integer) : Integer; asmname '_p_var_anystring_setlength';
procedure AnyStringTFDD_Rewrite (var f : AnyFile; var Buf : VarAnyString); asmname '_p_anystring_tfdd_rewrite';
procedure StringTFDD_Rewrite (var f : AnyFile; var Buf : VarAnyString; var s : String); asmname '_p_string_tfdd_rewrite';

function  GPC_UpCase      (ch : Char) : Char;                                    asmname '_p_gpc_upcase';
function  GPC_LoCase      (ch : Char) : Char;                                    asmname '_p_gpc_locase';
function  BP_UpCase       (ch : Char) : Char;                                    asmname '_p_bp_upcase';
function  BP_LoCase       (ch : Char) : Char;                                    asmname '_p_bp_locase';

{ Internal routines for the string operators }

{ Compare strings for equality without padding }
function  StrEQ (s1 : PChar; Length1 : Integer; s2 : PChar; Length2 : Integer) : Boolean;    asmname '_p_eq';

{ Compare strings for `less-than' without padding }
function  StrLT (s1 : PChar; Length1 : Integer; s2 : PChar; Length2 : Integer) : Boolean;    asmname '_p_lt';

{ Compare strings for equality, padding the shorter string with spaces }
function  StrEQPad (s1 : PChar; Length1 : Integer; s2 : PChar; Length2 : Integer) : Boolean; asmname '_p_str_eq';

{ Compare strings for `less-than', padding the shorter string with spaces }
function  StrLTPad (s1 : PChar; Length1 : Integer; s2 : PChar; Length2 : Integer) : Boolean; asmname '_p_str_lt';

{ Internal routine for Index/Pos }
function  GPC_Index (s1 : PChar; Length1 : Integer; s2 : PChar; Length2 : Integer) : Integer; asmname '_p_index';

{ Internal routine for Trim }
function  GPC_Trim (Src : PChar; SrcLength : Integer; Dest : PChar) : Integer; asmname '_p_trim';

{ Internal routine for SubStr/Copy }
function  GPC_SubStr (Src : PChar; SrcLength, aPosition, Count : Integer; Dest : PChar; TruncateFlag : Boolean) : Integer; asmname '_p_substr';

procedure GPC_Insert      (const Source: String; var Dest : String;
                           aIndex : Integer; TruncateFlag : Boolean);            asmname '_p_insert';
procedure GPC_Delete      (var s : String; aIndex, Count : Integer);             asmname '_p_delete';

type
  GPC_TimeStamp = {@@packed} record
                    DateValid,
                    TimeValid   : Boolean;
                    Year        : Integer;
                    Month       : 1 .. 12;
                    Day         : 1 .. 31;
                    DayOfWeek   : 0 .. 6;  { 0 means Sunday }
                    Hour        : 0 .. 23;
                    Minute      : 0 .. 59;
                    Second      : 0 .. 59;
                    MicroSecond : 0 .. 999999
                  end;

procedure GPC_GetTimeStamp (var aTimeStamp : TimeStamp);                                                    asmname '_p_gettimestamp';
procedure GPC_Date (protected var aTimeStamp : TimeStamp; var Dest : DateString);                            asmname '_p_date';
procedure GPC_Time (protected var aTimeStamp : TimeStamp; var Dest : TimeString);                            asmname '_p_time';

type
  { The standard fields are Bound and Name. The others are extensions. }
  GPC_BindingType = {@@packed} record
                      Bound             : Boolean;
                      Force             : Boolean;      { Can be set to allow binding to non-writable files or directories }
                      Extensions_Valid  : Boolean;
                      Readable          : Boolean;
                      Writable          : Boolean;
                      Executable        : Boolean;
                      Existing          : Boolean;      { Binding points to an existing file }
                      Directory         : Boolean;      { Binding points to an existing directory; `Existing' is False then }
                      Size              : LongInt;      { Number of elements or -1 }
                      AccessTime,                       { Time of last access }
                      ModificationTime,                 { Time of last modification }
                      ChangeTime        : UnixTimeType; { Time of last inode change }
                      User,                             { User ID of owner }
                      Group,                            { Group ID of owner }
                      Mode,                             { Protection mode, cf. ChMod }
                      INode             : Integer;      { Unix INode number }
                      CFile             : CFilePtr;     { allows binding a Pascal file to a C file }
                      Name              : String (Binding_Name_Length)
                    end;

procedure GPC_Bind         (          var aFile : AnyFile; protected var aBinding : BindingType); asmname '_p_bind';
procedure GPC_Unbind       (          var aFile : AnyFile);                                       asmname '_p_unbind';

var
  GPC_FileMode : asmname '_p_filemode' Integer;

procedure GPC_Assign  (var T : AnyFile; const Name : String); asmname '_p_assign';

procedure GPC_Flush    (          var aFile : AnyFile);                                        asmname '_p_flush';
function  GPC_GetFile  (protected var aFile : AnyFile) : CFilePtr;                             asmname '_p_getfile';

{ Various other versions of Reset, Rewrite and Extend are still overloaded magically }
procedure GPC_Rewrite  (          var aFile : AnyFile; aFileName : CString; aLength : Integer); asmname '_p_rewrite';
procedure GPC_Extend   (          var aFile : AnyFile; aFileName : CString; aLength : Integer); asmname '_p_extend';
procedure GPC_Reset    (          var aFile : AnyFile; aFileName : CString; aLength : Integer); asmname '_p_reset';
procedure GPC_Close    (          var aFile : AnyFile);                                        asmname '_p_close';
procedure GPC_Erase    (          var aFile : AnyFile);                                        asmname '_p_erase';
procedure GPC_Rename   (          var aFile : AnyFile; NewName : CString);                     asmname '_p_rename';
Procedure GPC_ChDir    (Path : CString);                                                       asmname '_p_chdir';
Procedure GPC_MkDir    (Path : CString);                                                       asmname '_p_mkdir';
Procedure GPC_RmDir    (Path : CString);                                                       asmname '_p_rmdir';
procedure GPC_Erase    (          var aFile : AnyFile);                                        asmname '_p_erase';
procedure GPC_Rename   (          var aFile : AnyFile; NewName : CString);                     asmname '_p_rename';
Procedure GPC_ChDir    (Path : CString);                                                       asmname '_p_chdir';
Procedure GPC_MkDir    (Path : CString);                                                       asmname '_p_mkdir';
Procedure GPC_RmDir    (Path : CString);                                                       asmname '_p_rmdir';

{ Random access file routines, from randfile.c }

function  GPC_GetSize      (          var aFile : AnyFile) : Integer;           asmname '_p_getsize';
procedure GPC_Truncate     (          var aFile : AnyFile);                     asmname '_p_truncate';
procedure GPC_DefineSize   (          var aFile : AnyFile; NewSize :  Integer); asmname '_p_definesize';
procedure GPC_SeekAll      (          var aFile : AnyFile; NewPlace : Integer); asmname '_p_seekall';
procedure GPC_SeekRead     (          var aFile : AnyFile; NewPlace : Integer); asmname '_p_seekread';
procedure GPC_SeekWrite    (          var aFile : AnyFile; NewPlace : Integer); asmname '_p_seekwrite';
procedure GPC_SeekUpdate   (          var aFile : AnyFile; NewPlace : Integer); asmname '_p_seekupdate';
function  GPC_Empty        (protected var aFile : AnyFile) : Boolean;           asmname '_p_empty';
procedure GPC_Update       (          var aFile : AnyFile);                     asmname '_p_update';
function  GPC_LastPosition (          var aFile : AnyFile) : Integer;           asmname '_p_lastposition';
function  GPC_Position     (          var aFile : AnyFile) : Integer;           asmname '_p_position';

{ Versions with only 3 parameters are still overloaded magically }
(*@@*)procedure GPC_BlockRead  (var aFile : File;           var Buf : Void; Count : Cardinal; var BytesRead    : Cardinal); asmname '_p_blockread';
(*@@*)procedure GPC_BlockWrite (var aFile : File; protected var Buf : Void; Count : Cardinal; var BytesWritten : Cardinal); asmname '_p_blockwrite';

{ Routines to output various things, from write.c }

procedure GPC_Page      (var aFile : Text);                       asmname '_p_page';
procedure GPC_Put       (var aFile : AnyFile);                    asmname '_p_put';

{ Int and Frac routines for real numbers, from math.pas }

function GPC_Frac (x : LongestReal) : LongestReal; asmname '_p_frac';
function GPC_Int  (x : LongestReal) : LongestReal; asmname '_p_int';

{ Transcendental functions for Reals and LongReals, from maths.c }

function Real_Arctan     (x : Double)                : Double;   asmname '_p_arctan';
function Real_Sqrt       (x : Double)                : Double;   asmname '_p_sqrt';
function Real_Ln         (x : Double)                : Double;   asmname '_p_ln';
function Real_Exp        (x : Double)                : Double;   asmname '_p_exp';
function Real_Sin        (x : Double)                : Double;   asmname '_p_sin';

function Real_Cos        (x : Double)                : Double;   asmname '_p_cos';
function LongReal_Arctan (x : LongReal)              : LongReal; asmname '_pp_arctan';
function LongReal_Sqrt   (x : LongReal)              : LongReal; asmname '_pp_sqrt';
function LongReal_Ln     (x : LongReal)              : LongReal; asmname '_pp_ln';
function LongReal_Exp    (x : LongReal)              : LongReal; asmname '_pp_exp';
function LongReal_Sin    (x : LongReal)              : LongReal; asmname '_pp_sin';
function LongReal_Cos    (x : LongReal)              : LongReal; asmname '_pp_cos';

{ Extended Pascal `**' operator }
function Real_Power      (x, y : Double)             : Double;   asmname '_p_expon';
function LongReal_Power  (x, y : LongReal)           : LongReal; asmname '_pp_expon';

{ from math.pas }

const
  GPC_Pi = 3.14159265358979323846264338327950288419716;

function Real_Pow        (x : Double; y : Integer)   : Double;   asmname '_p_pow';
function LongReal_Pow    (x : LongReal; y : Integer) : LongReal; asmname '_pp_pow';

{ Library functions for complex type arguments, from math.pas }

function Complex_Polar  (aLength, Theta : Double)  : Complex; asmname '_p_polar';
function Complex_Arg    (z : Complex)              : Double;  asmname '_p_arg';
function Complex_Arctan (z : Complex)              : Complex; asmname '_p_z_arctan';
function Complex_Sqrt   (z : Complex)              : Complex; asmname '_p_z_sqrt';
function Complex_Ln     (z : Complex)              : Complex; asmname '_p_z_ln';
function Complex_Exp    (z : Complex)              : Complex; asmname '_p_z_exp';
function Complex_Sin    (z : Complex)              : Complex; asmname '_p_z_sin';
function Complex_Cos    (z : Complex)              : Complex; asmname '_p_z_cos';
function Complex_Pow    (z : Complex; y : Integer) : Complex; asmname '_p_z_pow';
function Complex_Power  (z : Complex; y : Double)  : Complex; asmname '_p_z_expon';

{ Random number routines, from random.c }

{ RandomizePtr, RandRealPtr and RandIntPtr point to these routines by default }
procedure Default_Randomize;                                 asmname '_p_default_randomize';
function  Default_RandReal : LongestReal;                    asmname '_p_default_randreal';
function  Default_RandInt (MaxValue : LongestCard) : LongestCard; asmname '_p_default_randint';

{ GPC_Randomize, GPC_RandReal and GPC_RandInt call the actual routines through RandomizePtr, RandRealPtr and RandIntPtr }
procedure GPC_Randomize;                                 asmname '_p_randomize';
function  GPC_RandReal : LongestReal;                    asmname '_p_randreal';
function  GPC_RandInt (MaxValue : LongestCard) : LongestCard; asmname '_p_randint';

implementation

end.
